//----Figure 16.4
//---Constants  
  //--Action
  NOACTION = 0
  EAT      = 1
  SLEEP    = 2
  PLAY     = 3
  EXPLORE  = 4
  SAVE     = 5
  RETREAT  = 6
  RESPOND  = 7

  //--Status
  HUNGRY   = 0
  SLEEPY   = 1
  BORED    = 2

  //--Feeling
  PAIN     = 1
  PLEASURE = 2

  //--Headings
  NORTH    = 0
  EAST     = 1
  SOUTH    = 2
  WEST     = 3

  NEEDS_THRESHOLD = 85
  EAT_AREA        = 8
  SLEEP_AREA_1    = 20
  SLEEP_AREA_2    = 26
  PLAY_AREA       = 0
  PAIN_AREA_1     = 11
  PAIN_AREA_2     = 29
  PAIN_AREA_3     = 25
//===============================================================
//=============================================================
MainProgram:
  GoSub DisplayInstructions
  GoSub InitializeSimulation
  if not Key then  GoSub RestoreMemory
  GoSub ComeToLife
End
//=============================================================
//=============================================================
//--- Subroutine RestoreMemory
//--- Inputs  : none
//--- Outputs : Memory[],BadList[],MemPtr,BadPtr
//--- Calls To: none
//---
//--- Reads the arrays Memory[] and BadList[] from files 
//--- created by previous runs.
//-------------------------------------------------------------
RestoreMemory:
   if FilExists("MemoryGood") and FilExists("MemoryBad")
      MRead Memory,"MemoryGood"
      MRead BadList,"MemoryBad"
      MemPtr = Memory[99,0]
      BadPtr = BadList[99,0]
   endif
Return
//=============================================================
//=============================================================
//--- Subroutine ComeToLife
//--- Inputs  : none
//--- Outputs : none
//--- Calls To: CheckMemory,DisplayAction,DoMovement
//---           CheckBadList
//--- Makes the robot do SOMETHING either based on memory of 
//--- past experiences or some new random choice. The robot's
//--- memory is altered as the robot encounters pain and 
//--- pleasure.
//-------------------------------------------------------------
ComeToLife:
  while true
   gosub CheckMemory
   if HaveResponse
     Action = RESPOND
     gosub DisplayAction
     Movement = Memory[HaveResponse,2]
     gosub DoMovement
   else
     if random(50)>=20 then Movement = Random(4)
     gosub CheckBadList
     if not Bad
       Action=EXPLORE
       gosub DisplayAction
       gosub DoMovement
       if Status = PAIN
         // Save to bad list
         BadList[BadPtr,0]=LastCell
         BadList[BadPtr,1]=Movement
         BadPtr = BadPtr+1
       elseif Status = PLEASURE
         //Save to Memory if NOT there already
         AddIt = True
         if MemPtr>0
           for i=0 to MemPtr-1
            if Memory[i,0]=CurNeed and Memory[i,1]=LastCell
              AddIt = False
              break
            endif
           next
         endif
         if AddIt
           Memory[MemPtr, 0] = CurNeed
           Memory[MemPtr, 1] = LastCell
           Memory[MemPtr, 2] = Movement
           if (MemPtr=25) or (MemPtr=50) or (MemPtr=75)
             Memory[99,0]=MemPtr+1
             BadList[99,0]=BadPtr
             MWrite Memory,"MemoryGood"
             MWrite BadList,"MemoryBad"
           endif
           MemPtr = MemPtr+1
         endif
       endif
     else
       Action=RESPOND
       gosub DisplayAction
       Delay 200
     endif
   endif
  wend
Return
//=============================================================
//=============================================================
//--- Subroutine DoMovement
//--- Inputs  : Movement,MemPtr,BadPtr,CurCell,Needs[],Rx,Ry
//--- Outputs : Needs[],LastCell,Action,Status,CurCell
//--- Calls To: DispStatusMeter,DisplayAction,CheckStatus
//---
//--- Moves the robot based on the variable Movement the
//--- robot retreats from painful events, which are reported
//--- through the variable Status
//-------------------------------------------------------------
DoMovement:
  SetColor LightGreen
  xyString 150,15,"# Good Mem ",MemPtr
  xyString 300,15,"# Bad Mem ",BadPtr
  Status=0
  LastCell = CurCell // save where we came from
  // Increase Needs
  Needs[HUNGRY] = Needs[HUNGRY]+1
  if random (50)>25 then Needs[SLEEPY] = Needs[SLEEPY] + 1
  Needs[BORED] = Needs[BORED] + 1
  For DM_Which = 0 to 2
    gosub DispStatusMeter
  Next
  while rCompass() <> 90*Movement
    rTurn 90
  wend
  for i=1 to 120
    rForward 1
    if rBumper()
      Action = RETREAT
      Gosub DisplayAction
      rForward -i
      Status = PAIN
      return
    endif
  next
  dRx = 0 \ dRy = 0
  if Movement = EAST  then dRx = 1 
  if Movement = WEST  then dRx = -1
  if Movement = NORTH then dRy = -1 
  if Movement = SOUTH then dRy = 1 
  Rx = Rx+dRx \ Ry = Ry+dRy
  CurCell = 5*Rx+Ry
  if CurCell=PAIN_AREA_1 or CurCell=PAIN_AREA_2 or CurCell=PAIN_AREA_3
    Status = PAIN
    Action = RETREAT
    Gosub DisplayAction
    rForward -120
    Rx = Rx-dRx \ Ry = Ry-dRy
  endif
  gosub CheckStatus
return   
//=============================================================
//=============================================================
//--- Subroutine DisplayAction
//--- Inputs  : Action,ACTIONS[]
//--- Outputs : none
//--- Calls To: none
//---
//--- Displays the current action in the actions list
//-------------------------------------------------------------
DisplayAction:
  SetColor Black
  xyString 735,440,"ACTION"
  line 733,457,790,457
  SetColor Yellow
  For DA_I = 1 to 7
     xystring 728,450+15*DA_I,ACTIONS[DA_I]
  next
  if Action <> NOACTION
    Setcolor Blue
    xyString 728,450+15*Action,ACTIONS[Action]
  endif
return
//=============================================================
//=============================================================
//--- Subroutine DispStatusMeter
//--- Inputs  : DM_Which,Needs[]
//--- Outputs : none
//--- Calls To: none
//---
//--- Updates the meters display
//-------------------------------------------------------------
DispStatusMeter:
  DM_Incr = DM_Which*140
  if Needs[DM_Which] > 100 then Needs[DM_Which] = 100
  DM_Value = 100-Needs[DM_Which]
  rectangle 732,50+DM_Incr,793,149+DM_Incr,white,white
  rectangle 732,50+DM_Value+DM_Incr,793,149+DM_Incr,Cyan,Cyan
  line 732,65+DM_Incr,793,65+DM_Incr,3,Red
return
//=============================================================
//=============================================================
//--- Subroutine CheckStatus
//--- Inputs  : Status,CurCell,Memory[],MemPtr,CurNeed,Needs[]
//--- Outputs : Status
//--- Calls To: DispStatusMeter,DisplayAction
//---
//--- Updates the status of the robot and displays the action
//--- being taken
//-------------------------------------------------------------
CheckStatus:
  if Status = PAIN then return
  // Sets Status if Pleasure
  Status = 0
  DM_Which = -1
  if (CurCell=EAT_AREA) and (CurNeed=EAT)
    Action = EAT
    gosub DisplayAction
    Needs[SLEEPY] = Needs[SLEEPY] + 15
    Needs[BORED]  = Needs[BORED]  + 10
    DM_Which = HUNGRY
  elseif (CurCell=SLEEP_AREA_1 or CurCell=SLEEP_AREA_2) and CurNeed=SLEEP
    Action = SLEEP
    gosub DisplayAction
    Needs[HUNGRY] = Needs[HUNGRY]+5
    Needs[BORED]  = Needs[BORED]+20
    DM_Which = SLEEPY
  elseif (CurCell=PLAY_AREA) and (CurNeed=PLAY)
    Action = PLAY
    gosub DisplayAction
    Needs[HUNGRY] = Needs[HUNGRY]+20
    Needs[SLEEPY] = Needs[SLEEPY] + 20
    DM_Which = BORED
  elseif MemPtr>0 
    for i=0 to MemPtr-1
      if (Memory[i,0]=CurNeed) and (Memory[i,1]=CurCell)
        // this cell is on the good list
        Status = PLEASURE
        return
      endif
    next
  endif
  if DM_Which < 0 then return
  for CS_I=Needs[DM_Which] to 0 step -1
    Needs[DM_Which] = CS_I
    gosub DispStatusMeter
    delay 50
  next
  Status = PLEASURE
  For DM_Which = 0 to 2
     gosub DispStatusMeter
  Next
return
//=============================================================
//=============================================================
//--- Subroutine CheckMemory
//--- Inputs  : Memory[],MemPtr,Needs[],CurCell
//--- Outputs : CurNeed,HaveResponse
//--- Calls To: none
//---
//--- Uses the biological needs to cause reflexive actions
//--- such as eating and sleeping. Other needs are affected by
//--- these actions.
//-------------------------------------------------------------
CheckMemory:
  // First Establish Current Need
  CurNeed = 0
  Maximum = Needs[HUNGRY]
  for CM_I = SLEEPY to BORED
     if Needs[CM_I] > Maximum then Maximum=Needs[CM_I]
  next
  if Maximum > NEEDS_THRESHOLD
     if Maximum = Needs[BORED]  then CurNeed = PLAY
     if Maximum = Needs[SLEEPY] then CurNeed = SLEEP
     if Maximum = Needs[HUNGRY] then CurNeed = EAT
     //Hunger is higher order priority than sleep and 
     //sleep is higher order than play
  endif
  // sets variable HaveResponse to position of match
  // in memory or zero (false) if no match is found
  HaveResponse = 0
  if MemPtr=0 then return
  for i=0 to MemPtr-1
    if (CurNeed=Memory[i,0]) and (CurCell=Memory[i,1])
      HaveResponse=i // shows where found
      break
    endif
  next  
return
//=============================================================
//=============================================================
//--- Subroutine CheckBadList
//--- Inputs  : BadPtr,BadList[],CurCell,Movement
//--- Outputs : Bad
//--- Calls To: none
//---
//--- Checks to see if a proposed move from a cell is in the
//--- bad move memory
//-------------------------------------------------------------
CheckBadList:
  // sets variable BAD
  Bad=false
  if BadPtr=0 then return
  for i=0 to BadPtr-1
    if (BadList[i,0]=CurCell) and (BadList[i,1]=Movement)
      Bad = True
      break
    endif
  next
return
//=============================================================
//=============================================================
//--- Subroutine DisplayInstructions
//--- Inputs  : none
//--- Outputs : Key
//--- Calls To: none
//---
//--- Displays instruction and waits for a key press then 
//--- returns the character pressed in the variable Key
//-------------------------------------------------------------
DisplayInstructions:
  data MI;"Figure16.04.Bas"
  data MI;"This program transforms the robot into a 'living' creature."
  data MI;"It has needs as hunger, boredom, and even the need to sleep."
  data MI;"These needs have priorities and only the largest current need"
  data MI;"is a motivator.  Like a biological creature, there are inborn"
  data MI;"characteristics such as curiosity and a goal to satisfy the "
  data MI;"internal needs.",""
  data MI;"When a creature is born, all of the needs are present. The"
  data MI;"creature will explore and if a need is satisfied (by accident)"
  data MI;"the meters will reflect the fact.  The creature will also"
  data MI;"remember things that give it pain or pleasure and act differently"
  data MI;"in the future because of its memory."
  data MI;"In the beginning the creature seems to roam aimlessly. However,"
  data MI;"after an hour or so, the creature will have explored enough to "
  data MI;"have a good amount of memory.  At that time you will see that "
  data MI;"the creature has learned how to satisfy needs that motivate it."
  data MI;"At that time, it will only roam when all the needs are low as"
  data MI;"indicated by the graphs.  See the text for more information.",""
  data MI;"You can save and retrieve the memory if you wish. Currently, "
  data MI;"the program will automatically save its memory (in the files "
  data MI;"MEMORYGOOD and MEMORYBAD) when the memory size is 25, 50, and 75.",""
  data MI;"If memory has been saved, you can now load it by pressing the"
  data MI;"'Cancel' button or Esc key. Pressing 'OK' or Enter key will start"
  data MI;"the program with no initial memory and will build a new memory."
  Key = MsgBox(MI)
return
//=============================================================
//=============================================================
//--- Subroutine InitializeSimulation
//--- Inputs  : none
//--- Outputs : Needs[],BadPtr,MemPtr,CurNeed,
//---           CurCell,LastCell,Rx,Ry
//--- Calls To: none
//---
//--- Draws the environment and initializes all variables
//--- and arrays.
//-------------------------------------------------------------
InitializeSimulation:
  //--- Initialize system variables
  Data ACTIONS;"NO ACTION","  EAT"," SLEEP"," PLAY","EXPLORE"
  Data ACTIONS;" SAVE","RETREAT","RESPOND"
  Dim Needs[3]
  MConstant Needs,100 //all needs are maximum
  Dim Memory[100,3] // Memory saves CurNeed,CurCell,Movement
  MemPtr=0
  Dim BadList[100,2]  // BadList does not include needs
  Action = NOACTION \ Status = HUNGRY \ Movement = NORTH
  BadPtr=0 \ CurNeed=0 \ CurCell=0 \ LastCell=0

  //---Draw Environment
  //----outside border
  SetColor Black
  LineWidth 3
  rectangle 0,0,720,595
  //----Sleep area
  LineWidth 5
  circle 500,-220,940,220,LightGreen,LightGreen
  circle 580,-140,860,140,LightGreen
  SetColor Black,LightGreen
  xyString 520,2, "SLEEP"
  xyString 525,16,"AREA"
  LineWidth 3
  Rectangle 720,0,800,595,black,white
  SetColor Black
  line 0,0,720,0
  //---- Activity Area
  SetColor Black,Yellow
  rectangle 3,3,120,120,Yellow,Yellow
  xyString 3,2,"ACTIVITY AREA"
  //---- Fire area
  LineWidth 5
  SetColor Red
  data FirePlace;-600,0, 720,0, 720,120, 690,30, 600,0, 670,-3
  MPolygon FirePlace,Red
  SetColor Black,Red
  xyString 670,3,"FIRE"
  //----Walls
  SetColor Black,White
  data FoodArea; -240,360, 240,480, 120,480, 120,240, 360,240
  data FoodArea;  360,360, 480,360, 480,480, -360,480, 360,600
  MPolygon FoodArea
  xyString 127,460," FOOD AREA"
  //----Briar areas
  SetColor Black,LightGreen
  rectangle 240,120,360,236,LightGreen,LightGreen
  xyString 262,218," BRIARS "
  rectangle 600,480,716,591,LightGreen,LightGreen
  xyString 622,573," BRIARS "
  //----meters
  LineWidth 2
  SetColor Black,White
  xyString 740,4,"NEEDS"
  line 735,22,786,22
  rectangle 730,30,795,150
  xyString 735,32,"Hungry"
  rectangle 730,170,795,290
  xyString 735,172,"Sleepy"
  rectangle 730,310,795,430
  xyString 740,315,"Bored"
  rectangle 732,50,793,149,Cyan,Cyan // Hungry
  rectangle 732,190,793,289,Cyan,Cyan // Sleepy
  rectangle 732,330,793,429,Cyan,Cyan // Bored 
  //---Place the Robot
  rLocate 420,300
  Rx = 3 \ Ry = 2 \ CurCell = 17
  rInvisible LightGreen,Yellow
return
//=============================================================
