// This DEMO was discussed in a SERVO article by Blankenship/Mishal
// It creates a 3D arm that can perform simple tasks as requested through buttons

MainProgram:
  gosub Initialize
  // replace the following line with your code
  // if you want to create your own programs
  gosub IntelligentDemo
end
//===================================================================
Initialize:
  OS=160// offset to move the arm up for netbook use (low resolution screen)
  Dim Arm[10,5]
  Dim nArm [10,5]
  Dim Grid[20,5]
  Dim Object1[4,5]
  Dim Object2[4,5]
  Dim Object3[4,5]
  Dim ObjectCenter[4,2]
  Dim OO[12,2]
  // object position array
  Dim ObjPos[3,2]
  for i=0 to 2
    ObjPos[i,0]=200
    ObjPos[i,0]=0
  next
  Data oCol; 23456,Red,Green,Yellow
  // arm
  Data Points; -10,-20,0,0,0, -10,20,0,0,0,  25,0,0,0,0,  100,0,20,0,0, 200,0,20,0,0, 200,0,0,0,0
  // object in hand
  Data Points; 180,-10,0,0,0,  180,10,0,0,0,  220,10,0,0,0, 220,-10,0,0,0
  mCopy Points,Arm
  // grid pints
  Data gPoints;  300,-200,0,0,0,  300,-100,0,0,0,   300,0,0,0,0,  300,100,0,0,0,  300,200,0,0,0
  Data gPoints; -200,300,0,0,0,  -100,300,0,0,0,    0,300,0,0,0,  100,300,0,0,0,  200,300,0,0,0
  Data gPoints; -300,-200,0,0,0, -300,-100,0,0,0,  -300,0,0,0,0, -300,100,0,0,0, -300,200,0,0,0 
  Data gPoints; -200,-300,0,0,0, -100,-300,0,0,0,   0,-300,0,0,0, 100,-300,0,0,0, 200,-300,0,0,0
  mCopy gPoints,Grid
  //objects (current positions)
  Data oPoints1; 180,-10,0,0,0,  180,10,0,0,0,  220,10,0,0,0,  220,-10,0,0,0
  Data oPoints2; 190,-20,0,0,0,  190,20,0,0,0,  210,20,0,0,0,  210,-20,0,0,0
  Data oPoints3; 225,0,0,0,0,    200,25,0,0,0,  175,0,0,0,0,   200,-25,0,0,0
  mCopy oPoints1, Object1
  mCopy oPoints2, Object2
  mCopy oPoints3, Object3
  //object offsets (OO)
  Data OOP; -20,-10,  -20,10, 20,10,   20,-10
  Data OOP; -10,-20,  -10,20, 10,20,   10,-20
  Data OOP; 20,0,     0,20,    -20,0,  0,-20  
  mCopy OOP,OO
  Data View; 350,.01,1,300,400,300
  Flip on
  ClearScr gray
  // draw grid
  ge3Dto2DA Grid,View
  for i=0 to 9
    Line Grid[i,3],Grid[i,4]-OS,Grid[i+10,3],Grid[i+10,4]-OS,1,DarkGray
  next
  SetColor Black,Gray
  xyString 560,20,"Grid lines are spaced"
  xyString 580,40,"at 100 pixels."
  //Line 0,432,799,432,5,RED // bottom of low resolution screen
  SaveScr
  flip
  // system variables
  FT1=True
  FT2=True
  FT3=True
  ArmErr = False
  sAngle = DtoR(60) \ fAngle = DtoR(-20) \ angle = DtoR(90)  // initial arm positions
  InHand = 0 // indicates object being held
  // Control variables (available to user)
  Data Order;3,2,1  // order to draw objects based on position
  Base = 0 \ Shoulder=0 \ Elbow = 0 // set these before calling MoveArm
  Vac = False            // set this to pick up object
  ShoulderAngle = RtoD(sAngle) // angles may be read to determine Arm's  current position
  ElbowAngle = RtoD(fAngle)
  BaseAngle = RtoD(angle)
  // draw objects in initial position
  gosub DrawObject1
  gosub DrawObject2
  gosub DrawObject3
  gosub DrawArm
  flip
return
//===================================================================
Rot:
  for ir= 0 to 9
    geRotVz Arm[ir,0],Arm[ir,1],Arm[ir,2],angle,a,b,c 
    nArm[ir,0]=a
    nArm[ir,1]=b
    nArm[ir,2]=c
  next
return   
//===================================================================
DrawCircle:
   // assume center cx,cy
   cr=5
   circle cx-cr,cy-cr,cx+cr,cy+cr,black,black
return
//===================================================================
MoveJoints:
  // adjust first 2 arm segments
  Arm[3,0]=100*cos(sAngle)
  Arm[3,2]=100*sin(sAngle)+20
  Arm[4,0]=100*cos(fAngle)+Arm[3,0]
  Arm[4,2]=100*sin(fAngle)+Arm[3,2]
  // adjust third segment
  Arm[5,0]=Arm[4,0] \ Arm[5,2]=Arm[4,2]-18
  if InHand = 1
       Arm[6,0]=Arm[4,0]-20 \ Arm[7,0]=Arm[6,0] \ Arm[8,0]=Arm[4,0]+20 \ Arm[9,0]=Arm[8,0]
       Arm[6,2]=Arm[5,2] \ Arm[7,2]=Arm[5,2] \ Arm[8,2]=Arm[5,2] \ Arm[9,2]=Arm[5,2]
  elseif InHand = 2
       Arm[6,0]=Arm[4,0]-10 \ Arm[7,0]=Arm[6,0] \ Arm[8,0]=Arm[4,0]+10 \ Arm[9,0]=Arm[8,0]
       Arm[6,2]=Arm[5,2] \ Arm[7,2]=Arm[5,2] \ Arm[8,2]=Arm[5,2] \ Arm[9,2]=Arm[5,2]
  elseif InHand = 3
    Arm[6,0]=Arm[4,0]+25 \ Arm[7,0]=Arm[4,0] \ Arm[8,0]=Arm[4,0]-25 \ Arm[9,0]=Arm[4,0]
    Arm[6,2]=Arm[5,2] \ Arm[7,2]=Arm[5,2] \ Arm[8,2]=Arm[5,2] \ Arm[9,2]=Arm[5,2]
  endif
return
//===================================================================
DrawArm:
    // Assumes Arm[] contains the arms coordinates at angle zero
    //   angle = the base angle
    //    PlayWav "Retros"  // add a wave file here if you want sound as the arm moves
    gosub MoveJoints
    gosub Rot
    ge3Dto2DA nArm,View 
    RestoreScr
    for ii=0 to 2
      if Order[ii]=1 then gosub DrawObject1
      if Order[ii]=2 then gosub DrawObject2
      if Order[ii]=3 then gosub DrawObject3
    next
    // draw base
    Line nArm[0,3],nArm[0,4]-OS,nArm[2,3],nArm[2,4]-OS,2,black
    for ii = 1 to 0 
      LineTo nArm[ii,3],nArm[ii,4]-OS,2,black
    next    
    FloodFill2 400,300-OS,blue,black
    // draw lower arm
    Line 400,300-OS,400,280-OS,5,Black
    LineTo nArm[3,3],nArm[3,4]-OS,5,black
    cx=400 \ cy=280-OS \ gosub DrawCircle
    cx=nArm[3,3] \ cy=nArm[3,4]-OS \ gosub DrawCircle
    if InHand
      // draw object in hand
      Line nArm[6,3],nArm[6,4]-OS,nArm[9,3],nArm[9,4]-OS,1,oCol[0]
      for ii=8 to 6
        LineTo nArm[ii,3],nArm[ii,4]-OS,1,oCol[0]
      next
      FloodFill2 nArm[5,3],nArm[5,4]-OS,oCol[InHand],oCol[0]
    endif
    // draw rest of arm
    Line nArm[3,3],nArm[3,4]-OS,nArm[4,3],nArm[4,4]-OS,5,black
    LineTo nArm[5,3],nArm[5,4]-OS,5,black
    cx=nArm[4,3] \ cy=nArm[4,4]-OS \ gosub DrawCircle
    xyString 600,60,"BASE Angle = ",BaseAngle
    xyString 620,80,"SHOULDER Ang = ",ShoulderAngle
    xyString 640,100,"ELBOW Angle = ",ElbowAngle
    flip
return
//===================================================================
MoveArm:
  // change angles
  Change = DtoR(1)
  angle = angle+Base*Change
  sAngle = sAngle+Shoulder*Change
  // Shouder*Change is added to keep elbow relative to shoulder movement
  fAngle = fAngle+Elbow*Change + Shoulder*Change
  // check limits
  if angle > DtoR(150)
    beep
    angle=DtoR(150)
    ArmErr = True
  endif
  if angle < DtoR(-150)
    beep
    angle=DtoR(-150)
    ArmErr = True
  endif
  if sAngle > DtoR(90)
    beep
    sAngle = DtoR(90)
    ArmErr = True
  endif
  if sAngle<-2 // allow a little leeway at the limit
    beep
    sAngle=0
    ArmErr = True
  endif
  // check for object pickup or release
  if within(100*Sin(sAngle)+100*Sin(fAngle),0,3)// close to ground
    if (Vac=0) and (InHand>0)
      // release object
        for c=0 to 3 // copy 4 points 
          for cc= 3 to 4  // copy last two elements for each point
            if InHand = 1 then Object1[c,cc] = nArm[c+6,cc]
            if InHand = 2 then Object2[c,cc] = nArm[c+6,cc]  
            if InHand = 3 then Object3[c,cc] = nArm[c+6,cc]
          next
        next
      // change order to draw objects (makes stacks view properly
      for c= 0 to 2
        if Order[c]=InHand then break
      next
      if c<2
        for cc = c to 1
          Order[cc]=Order[cc+1]
        next
      endif  
      Order[2] = InHand         
      InHand=0
    endif   
    oNum=0  
    if Vac=True and InHand=False
      // pickup object if one is there
      //determine which object (set oNum)
      //all objects must be on ground to be here, so
      //check to see if objects center is close to end of arm
      //check for objects in order so that top object will be found first  
      for num=2 to 0
        if Within (ObjectCenter[Order[num],0]-nArm[5,3],-10,+10) and Within (ObjectCenter[Order[num],1]-nArm[5,4],-10,+10)
           // object found
           oNum = Order[num]
           break
        endif
      next       
      for c=0 to 3 // copy 4 points 
        for cc= 0 to 4  // copy 5 elements for each point
          if oNum = 1 then Arm[c+6,cc]  = Object1[c,cc]
          if oNum = 2 then Arm[c+6,cc]  = Object2[c,cc]
          if oNum = 3 then Arm[c+6,cc]  = Object3[c,cc]
        next
      next
      if oNum>0 then InHand=oNum
    endif
  endif
  // handle errors and report angles
  // allow a little leeway when arm moves to ground
  if 100*Sin(sAngle)+100*Sin(fAngle)<-2 then beep 
  ShoulderAngle = round(RtoD(sAngle))
  ElbowAngle = -1*round(RtoD(fAngle))
  BaseAngle = round(RtoD(angle))
  gosub DrawArm
return
//===================================================================
DrawObject1:
  if InHand=1 then return
  IF FT1 then ge3Dto2DA Object1,View \FT1=False
  sumX=0\sumY=0
  for jj=0 to 3
    sumX=sumX+Object1[jj,3]
    sumY=sumY+Object1[jj,4]
  next
  ObjectCenter[1,0] = sumX/4      // x
  ObjectCenter[1,1] = sumY/4      // y
  line Object1[0,3],Object1[0,4]-OS,Object1[3,3],Object1[3,4]-OS,2,oCol[0]+1
  for jj=2 to 0
     lineTo Object1[jj,3],Object1[jj,4]-OS,2,oCol[0]+1
  next
  FloodFill2 ObjectCenter[1,0],ObjectCenter[1,1]-OS,oCol[1],oCol[0]+1
return
//===================================================================
DrawObject2:
  if InHand=2 then return
  if FT2 then ge3Dto2DA Object2,View\FT2=FALSE
  sumX=0\sumY=0
  for jj=0 to 3
    sumX=sumX+Object2[jj,3]
    sumY=sumY+Object2[jj,4]
  next
  ObjectCenter[2,0] = sumX/4      // x
  ObjectCenter[2,1] = sumY/4      // y
  line Object2[0,3],Object2[0,4]-OS,Object2[3,3],Object2[3,4]-OS,2,oCol[0]+2
  for jj=2 to 0
     lineTo Object2[jj,3],Object2[jj,4]-OS,2,oCol[0]+2
  next
  FloodFill2 ObjectCenter[2,0],ObjectCenter[2,1]-OS,oCol[2],oCol[0]+2
return
//===================================================================
DrawObject3:
  if InHand=3 then return
  if FT3 then ge3Dto2DA Object3,View\FT3=False
  sumX=0\sumY=0
  for jj=0 to 3
    sumX=sumX+Object3[jj,3]
    sumY=sumY+Object3[jj,4]
  next
  ObjectCenter[3,0] = sumX/4      // x
  ObjectCenter[3,1] = sumY/4      // y
  line Object3[0,3],Object3[0,4]-OS,Object3[3,3],Object3[3,4]-OS,2,oCol[0]+3
  for jj=2 to 0
     lineTo Object3[jj,3],Object3[jj,4]-OS,2,oCol[0]+3
  next
  FloodFill2 ObjectCenter[3,0],ObjectCenter[3,1]-OS,oCol[3],oCol[0]+3
return
//===================================================================
// From this point forward, you will see a demo giving the arm some
// degree of intelligence. If you are programming the arm, you should
// NOT change any of the statements in the simulation program itself.
// Instead, you should write your control routines (as done here)
// and move the arm through a call to the subroutine MoveArm.
// Before the call, you should set three variables (Base, Shoulder,and
// Elbow) to -1 or 1 to move the respective joint forward or backward
// or 0 if you don't want that joint to move.  When the routine returns
// you can check the variable InHand to see if any object is held.
// InHand will equal 0 if hand is empty or 1,2,or 3 for the possible
// objects.  Objects are basically pieces of paper that can be stacked
// anywhere in the workspace.  They are picked up with a "vaccuum"
// which is turned on/off by setting the variable Vac to true/false.
// The arm will NOT pickup or release an object unless the vaccuum is
// very close to the paper itself and reasonably near the center.
// You can determine the current position of the arm by reading the 
// variables BaseAngle, ShoulderAngle, and ElbowAngle.
// You should observe the joint limits given below:
//    Base:     -150 to +150
//    Shoulder:    0 to   89
//    Elbow:   No checked limit
// If you exceed the limits or move the arm into the ground, a beep
// will sound.
//===================================================================
IntelligentDemo:
  // There are 4 fixed positions in this demo
  // Buttons are used to select the operation to perform
  // Arm will handle the operation with some intelligence
  gosub InitIntelArm
  gosub ProcessButtons
return
//===================================================================
InitIntelArm:
  Data CurPos; 0,0,0,0 // holds current postion of colors 1-3 (0 not used)
  //Data PosAngle; -60,-20,20,60
  Dim PosAngle[4]
  Dim JointAngle[4]
  // change the first two numbers (x,y) in each of the four groups below if
  // you wish to change the coordinates of the four positions where the
  // colored objects can be placed
  Data PosXY; 15,-180,0,0,0,   130,-90,0,0,0,   150,75,0,0,0,  50,150,0,0,0
  Dim BloPos[4,5]
  mCopy PosXY,BloPos

  
  // Calculate rotation angle and joint angle (same for both) for each position
  for i= 0 to 3
    PosAngle[i] = round(RtoD(atan(1.0*BloPos[i,1]/BloPos[i,0])))
    dist = (BloPos[i,0]*BloPos[i,0]+BloPos[i,1]*BloPos[i,1])^.5
    dist = dist/2
    s = (10000-dist*dist)^.5
    JointAngle[i]=round(Abs(RtoD(atan(s/dist))))
  next
  ge3Dto2DA BloPos,View
  AddButton " Initializing - Please Wait ",325,380
  AddButton "0",BloPos[0,3]+sign(PosAngle[0])*50,BloPos[0,4]-10-OS
  AddButton "1",BloPos[1,3]+sign(PosAngle[1])*50,BloPos[1,4]-10-OS
  AddButton "2",BloPos[2,3]+sign(PosAngle[2])*50,BloPos[2,4]-10-OS
  AddButton "3",BloPos[3,3]+sign(PosAngle[3])*50,BloPos[3,4]-10-OS    
    
  // move blocks to their starting positions    
  for i = 0 to 2
     AA=0\gosub RotToAngle
     // all the way down
     repeat
       while ShoulderAngle<>0
         Elbow=0\Shoulder=-1\Base=0       
         gosub MoveArm  
 
         while ElbowAngle<>0
           Elbow=1\Shoulder=0\Base=0       
           gosub MoveArm  
         wend
       wend   
     until ElbowAngle=0 and ShoulderAngle=0
     Vac = True
     gosub MoveArm
     gosub ArmUp
     position = i
     gosub MoveTo
     CurPos[InHand]= i
     gosub Drop
  next
  RemoveButton " Initializing - Please Wait "
  XOS=270
  AddButton " ON ",417-XOS,20
  AddButton " Yellow ",357-XOS,20
  curcol1=3
  AddButton " green  ",458-XOS,20
  curcol2=2
  AddButton " Pick Up ",290-XOS,20
  AddButton " MoveTo 0 ",20,70
  AddButton " MoveTo 1 ",20,100
  AddButton " MoveTo 2 ",20,130
  AddButton " MoveTo 3 ",20,160
  AddButton " Drop ",35,190
  Data msg; "   Instructions for Arm Simulation",""
  Data msg; "                 If you want these instructions again, just right click"
  Data msg; "               the mouse in the main window when the arm is NOT in motion.",""
  Data msg; "   The demo shows how nested modules can be used to easily create high level routines"
  Data msg; "   that appear to have some (small) amount of intelligence.  You can pick up a piece"
  Data msg; "   of colored paper (using a vaccuum) by specifying the color (to the right of PICK UP)."
  Data msg; "   You can move the paper to other locations using the MOVE TO and DROP buttons. If you"
  Data msg; "   choose two colors (click the colors to change them) you can use the ON button to"
  Data msg; "   make the arm place the first color ON the second color.  It will check to make sure"
  Data msg; "   there is nothing already in the hand, nothing on top of the destination color, and"
  Data msg; "   then move the first color ON the second regardless of how you have moved the papers."
  Data msg; "   This and MANY other projects will be explained in a new book 'Hardware Interfacing"
  Data msg; "   With RobotBASIC'. Visit wwww.RobotBASIC.com for more information on how to program"
  Data msg; "   this arm with your own programs. Examine the program and modify it to fix some of"
  Data msg; "   the idioscynrancies of this program (there are many things that can be improved)."
  Data msg; "   The book will show how to build real arms (and many other projects) and control them" 
  Data msg; "   with the same programs used to control the simulations, thus allowing you to"
  Data msg; "   experiment immediately without having to build or buy a real arm."
  delay 1000
  n=MsgBox (msg)
  // note: if there is an err in MsgBox (no n= for example) the error will be reported wrong    *************************************
  Elbow=0\Shoulder=0\Base=0
  gosub MoveArm
return
//===================================================================
// Below you will find a series of Arm Movement Primitives
// You can just call them to perform simple actions - and easy way to program the arm
//    or you can study them to see how to create your own actions
// The current angles of the joints are stored in variables such as ShoulderAngle and ElbowAngle
// Other variables such as Vac (tells you if the Vaccuum is on) and Hand (tells you what color
//    is currently in the hand) can also be tested to get information about the robot's state.
// You move the arm by setting variables like Shoulder, Elbow, and Base to 1(move forward),
//    -1(move backward), or 0(no movement) and then calling MoveArm to calculate and draw. 
// Some variables like CurPos[] and IsEmpty are not actually ARM variables but are variables
//    created and used in the demo (to keep track of parameters relating to the demo.
// This is definately a project for more advanced users, but we wanted to offer something 
//    special for that special reader that needs a challenge.  It also demonstrates some advanced
//    features such as Buttons etc.  ENJOY!
//===================================================================
ArmDown:
  // moves arm down by setting each angle to the one for this position
  while ShoulderAngle!=JointAngle[position] or ElbowAngle!=JointAngle[position]
    if ShoulderAngle!=JointAngle[position]
      if ShoulderAngle>JointAngle[position] then Shoulder=-1
      if ShoulderAngle<JointAngle[position] then Shoulder=1
      Elbow=0\Base=0
      gosub MoveArm
    endif
    if ElbowAngle!=JointAngle[position]
      if ElbowAngle>JointAngle[position] then Elbow=1
      if ElbowAngle<JointAngle[position] then Elbow=-1
      Shoulder=0\Base=0
      gosub MoveArm
    endif
  wend
return
//===================================================================
ArmUp:
  // moves arm till shoulder is vertical
  while ElbowAngle!=0 or ShoulderAngle<85
    Elbow=0\Shoulder=0\Base=0
    if ElbowAngle>0 then Elbow=1
    if ElbowAngle<0 then Elbow=-1
    if ShoulderAngle<85 then Shoulder=1
    gosub MoveArm
  wend
return
//===================================================================
Drop:
  // drop the block in hand at current angle (will lower arm to 25,-25)
  if !InHand then return
  gosub ArmDown
  CurPos[InHand]=position
  Vac = False
  gosub MoveArm
  gosub ArmUp
return
//===================================================================
PickUp:
  if InHand then gosub Discard
  gosub ArmDown
  Vac = True
  gosub MoveArm
  Elbow=0\Shoulder=0\Base=0
  gosub ArmUp
  if InHand>0 then CurPos[InHand]=4
return
//===================================================================
MoveTo:
  // move arm to angle specified by variable position
  AA = PosAngle[position]
  // when array above has a limit error, the error is reported incorrectly     ***************************************
  gosub RotToAngle
return
//===================================================================
MoveOver:
  // move arm over the block specified by variable block
  position =CurPos[block]
  gosub MoveTo
return
//===================================================================
Discard:
  // move block currently in hand to EMPTY position
  gosub FindEmpty
  position = Empty
  gosub MoveTo
  gosub Drop
return
//===================================================================
FindEmpty:
  for Empty=0 to 3
    IsEmpty = true  // assume this space is empty
    for fe = 1 to 3
      if CurPos[fe]=Empty 
        IsEmpty=false
        break
      endif
    next
    if IsEmpty then break // found an empty space
  next
return
//===================================================================
ColorOnColor:
  // places color curcol1 ON curcol2
  block = curcol2  // Make sure destination block is empty
  gosub PickUpColor
  gosub Drop
  block = curcol1
  gosub PickUpColor  // get the block to stack
  block = curcol2
  gosub MoveOver
  gosub Drop
return
//===================================================================
PickUpColor:
  //pickup block specified by variable block
  if InHand then gosub Discard
  while InHand<>block
    gosub MoveOver
    gosub PickUp
    if InHand<>block then gosub Discard
  wend
return
//===================================================================
ProcessButtons:
  while true
    ReadMouse ma,mb,mc
    if mc=2 then  n=MsgBox (msg)
    GetButton But
    if But = " ON "
      gosub ColorOnColor
    elseif But = " Pick Up "
      block = curcol1
      gosub PickUpColor
    elseif But = " Drop "
      gosub Drop
    elseif But = " Yellow "
      RemoveButton " Yellow "
      AddButton "  Red   ",357-XOS,20
      curcol1=1
    elseif But = " Green  "
      RemoveButton " Green  "
      AddButton " Yellow ",357-XOS,20
      curcol1=3    
    elseif But = "  Red   "
      RemoveButton "  Red   "
      AddButton " Green  ",357-XOS,20
      curcol1=2    
    elseif But = " yellow "
      RemoveButton " yellow "
      AddButton "  red   ",458-XOS,20
      curcol2=1    
    elseif But = " green  "
      RemoveButton " green  "
      AddButton " yellow ",458-XOS,20
      curcol2=3    
    elseif But = "  red   "
      RemoveButton "  red   "
      AddButton " green  ",458-XOS,20
      curcol2=2    
    elseif But = " MoveTo 0 "
      position = 0
      gosub MoveTo
    elseif But = " MoveTo 1 "
      position = 1
      gosub MoveTo    
    elseif But = " MoveTo 2 "
      position = 2
      gosub MoveTo    
    elseif But = " MoveTo 3 "
      position = 3
      gosub MoveTo
    endif
  Line 285-XOS,25,285-XOS,11,1
  LineTo 412-XOS,11
  LineTo 412-XOS,25
  Line 352-XOS,35,352-XOS,50
  LineTo 511-XOS,50
  LineTo 511-XOS,35
  flip
  wend
return
//===================================================================
RotToAngle:
  // moves base to angle AA
  while BaseAngle!=AA
    if BaseAngle!=AA       
      if BaseAngle>AA
        Base = -1
      elseif BaseAngle<AA
        Base = 1
      else
        Base = 0
      endif
      Shoulder=0
      Elbow=0
      gosub MoveArm
    endif
  wend
return
//===================================================================

// Try programming the arm yourself to do some simple applications
// You will find it has the same problems as programming a real arm
// You may even find it is far more difficult to control the arm than
// you might expect.  Don't be discourages though, you can learn a lot
// from playing with this program (and you don't have to buy or build 
// anything).   Note: A new version of RobotBASIC is due out May 2009
// that has ON EVENT capability.   Those features were not used in this
// program (even though they would have added a lot of functionality)
// because we wanted to maintain compatibility with the version
// currently available.  See www.RobotBASIC.com for more information.