// Robot puppet controller by John Blankenship & Samuel Mishal
// extended joystick required
// Parallax USB servo controller used to control 7 servos in puppet
// once devices are installed, note their assigned ports and alter
//the following lines if necessary.

//----Constants
  COMMS_PORT     = 1
  JOYSTICK_PORT  = 1
//====================================================================
MainProgram:
  GoSub Instructions
  GoSub SetUp
  GoSub Initialize
  GoSub Start_Control
  GoSub FinishUp
Exit
//====================================================================
FinishUp:
   For i=0 to MaxDim(Buttons,1)-1
      RemoveButton Buttons[i]
   Next
   ClearScr
   XYText 100,300,"Goodbye.......","Times New Roman",50,fs_Bold
   delay 1000
Exit
//====================================================================
SetUp:
  n=ErrMsg(Msgs[1],Msgs[0],MB_QUESTION|MB_YesNo|MB_Button2)
  if n=MB_Yes then GoSub Initialize_Comms
  GoSub Establish_Comms
  data Buttons;"&Instructions","&Quit"
  for i=0 to MaxDim(Buttons,1)-1
    AddButton Buttons[i],100+i*100,200,100,25
  next
  Quit = false
Return
//====================================================================
Initialize:
   // set these items to establish parameter for each action
   // Channel refers to the Parallax servo controller
   // LowLimit and HighLimit sets extent of servo momvement
   // Normal is the "centered" position
   // Randomness is the maximum amount of random servo movement 
   // Ramping effectively sets the speed 0-fast 10 or more-very slow

   // the following variables are used for random movements
   rArm = 0 \rTorso = 0 \rHeadR = 0 \ rHeadT = 0 
   
   //---Hat L/R buttons
   EyesChannel=14 \ EyesLowLimit=500 \ EyesHighLimit=800 
   EyesRamping=7 \ EyesNormal=650   \ EyesRandomness=0

   //--Button 1
   B_Mouth = 1
   MouthChannel=2 \ MouthLowLimit=650 \ MouthHighLimit=910 
   MouthRamping=1 \ MouthNormal=700   \ MouthRandomness=0

   //--Hat U Buttons
   H_Brows = 0
   BrowsChannel=4 \ BrowsLowLimit=550 \ BrowsHighLimit=800 
   BrowsRamping=7 \ BrowsNormal=700   \ BrowsRandomness=0

   //---R axis
   HeadRChannel=6 \ HeadRLowLimit=900 \ HeadRHighLimit=500 
   HeadRRamping=9 \ HeadRNormal=700   \ HeadRRandomness=30
   HeadRTime = timer()
   
   //---Y axis
   HeadTChannel=8 \ HeadTLowLimit=1000 \ HeadTHighLimit = 700 
   HeadTRamping=9 \ HeadTNormal=950   \ HeadTRandomness=50
   HeadTTime = timer()

   //---X axis
   TorsoChannel=10 \ TorsoLowLimit=620 \ TorsoHighLimit=800 
   TorsoRamping=15  \ TorsoNormal=720   \ TorsoRandomness=50
   TorsoTime = timer()

   //--Button 2
   B_Arms = 2
   ArmsChannel=12 \ ArmsLowLimit=300 \ ArmsHighLimit=700 
   ArmsRamping=10  \ ArmsNormal=700   \ ArmsRandomness=100
   ArmsTime = timer()
   
   //---Combination Movements Buttons
   B_Combo1 = 4 \ B_Combo2 = 8 \ B_Yes = 16 \ B_No = 32
Return
//====================================================================
Instructions:
  data Msgs;"Puppet.Bas","Do you want to intialize the controller board?"
  data I_M;Msgs[0],"The puppet control program"
  //n=MsgBox(I_M)
Return
//====================================================================
Initialize_Comms:
   // first set up communication to match the default for the 
   //    servo controller
   // note: This should only be done the first time the program is run
   // after the servo controller is turned on (which is why you are 
   // given the option of Initializing). But if the program is being run 
   // again without having turned off the servo controller then do not
   // choose this option.
   //---establish low speed communication
   setCommPort COMMS_PORT, br2400
   SerOut "!SCVER?",char(13)
   delay 200
   SerIn S
   // now set the Servo Controller's communication to maximum baud rate 
   //----set baud rate to 38400 baud
   SerOut "!SCSBR",char(1),char(13)
   delay 200
return
//====================================================================
Establish_Comms:
   //---establish high speed comms (set the PC to the higher baud rate)
   setCommPort COMMS_PORT, br38400
   SerOut "!SCVER?",char(13)
   delay 200
   SerIn S
Return
//====================================================================
Read_Joystick:
   GetButton btn
   if btn == Buttons[0] then n=MsgBox(I_M)
   if btn == Buttons[1] then Quit = true
   joystickE JOYSTICK_PORT,jvalue 
   jx = jvalue[0,0]/(jvalue[0,2]-jvalue[0,1])
   jy = jvalue[1,0]/(jvalue[1,2]-jvalue[1,1])
   jz = jvalue[2,0]/(jvalue[2,2]-jvalue[2,1])
   jr = jvalue[3,0]/(jvalue[3,2]-jvalue[3,1])
   jb = jvalue[6,0]
   jhat = jvalue[7,0]
   fmt="0.0000"
   xystring 0,0 ,"x=",format(jx,fmt)," y=",format(jy,fmt)," r=",format(jr,fmt)," z=",format(jz,fmt) 
   xystring 0,30,"Button=",format(jb,"0000"),"  Hat=",jhat,"   "
Return
//====================================================================
Start_Control:
   while !Quit
      GoSub Read_Joystick
      // check buttons first (special movement combinations
      if jb == B_Combo1 then GoSub Combo1 \ continue
      if jb == B_Combo2 then GoSub Combo2 \ continue
      if jb == B_Yes then GoSub Yes_Combo \ continue
      if jb == B_No then GoSub No_Combo \ continue
      // then move all motors based on joystick's position
      GoSub HeadR_Movements
      GoSub HeadT_Movements
      GoSub Arms_Movements
      GoSub Brows_Movements
      GoSub Eyes_Movements
      GoSub Mouth_Movements
      GoSub Torso_Movements
   wend
Return
//====================================================================
Combo1:
   m = "Combo 1"
   xyText 0,100,m+spaces(20),"",20,fs_Bold
   // add custom movements here
Return
//====================================================================
Combo2:
   m = "Combo 2"
   xyText 0,100,m+spaces(20),"",20,fs_Bold
   // add custom movements here
Return
//====================================================================
Yes_Combo:
   // example of how to do a combo movement
   m = "Yes"
   xyText 0,100,m+spaces(20),"",20,fs_Bold
   ramping = HeadTRamping   
   channel = HeadTChannel
   for i = 1 to 3
     nn = 750
     SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
     delay 500   
     nn = 950
     SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
     delay 500
   next
Return
//====================================================================
No_Combo:
   m = "No"
   xyText 0,100,m+spaces(20),"",20,fs_Bold
   ramping = HeadRRamping   
   channel = HeadRChannel
   for i = 0 to 3
     nn=600+i*25
     SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
     delay 500-i*50
     nn=800-i*25
     SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
     delay 500-i*50
   next
Return
//====================================================================
HeadR_Movements:
   // rotate the head
   ramping = HeadRRamping   
   channel = HeadRChannel
   if HeadRTime < Timer()
     rHeadR =random(HeadRRandomness)-HeadRRandomness/2
     HeadRTime = Timer()+1000 // random movement every 1000 mseconds
   endif
   nn = HeadRLowLimit+round(jr*(HeadRHighLimit-HeadRLowLimit))+rHeadR
   SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
Return

//====================================================================
HeadT_Movements:
   // tilt the head
   ramping = HeadTRamping   
   channel = HeadTChannel
   if HeadTTime < Timer()
     rHeadT =random(HeadTRandomness)-HeadTRandomness/2
     HeadTTime = Timer()+1000 // random movement every 1000 mseconds
   endif
   nn = HeadTLowLimit+round(jy*(HeadTHighLimit-HeadTLowLimit))+rHeadT
   SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
Return
//====================================================================
Arms_Movements:
   ramping = ArmsRamping
   channel = ArmsChannel
   if ArmsTime < Timer()
     rArms =random(ArmsRandomness)-ArmsRandomness/2
     ArmsTime = Timer()+3000 // random movment every 3000 mseconds
   endif
   if jb == B_Arms 
     n = ArmsHighLimit
   else 
     n = ArmsLowLimit + rArms
   endif   
   SerOut "!SC",char(channel),char(ramping),char(n&255),char((n >> 8)&255),char(13)
Return
//====================================================================
Eyes_Movements:
   ramping = EyesRamping
   channel = EyesChannel
   n = EyesNormal
   if jhat == 270 then n = EyesHighLimit
   if jhat == 90 then n = EyesLowLimit
   SerOut "!SC",char(channel),char(ramping),char(n&255),char((n >> 8)&255),char(13)
Return
//====================================================================
Brows_Movements:
   ramping = BrowsRamping
   channel = BrowsChannel
   n = BrowsNormal
   if jhat == 0 then n = BrowsHighLimit
   if jhat == 180 then n = BrowsLowLimit
   SerOut "!SC",char(channel),char(ramping),char(n&255),char((n >> 8)&255),char(13)
Return
//====================================================================
Mouth_Movements:
   ramping = MouthRamping
   channel = MouthChannel
   n = MouthLowLimit
   if jb == B_Mouth then n = MouthHighLimit
   SerOut "!SC",char(channel),char(ramping),char(n&255),char((n >> 8)&255),char(13)
Return
//====================================================================
Torso_Movements:
   ramping = TorsoRamping   
   channel = TorsoChannel
   if TorsoTime < Timer()
     rTorso =random(TorsoRandomness)-TorsoRandomness/2
     TorsoTime = Timer()+1000 // random movement every 1000 mseconds
   endif
   nn = TorsoLowLimit+round(jx*(TorsoHighLimit-TorsoLowLimit))+rTorso
   SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
Return
//====================================================================
