//====================================================================
//====================================================================
MainProgram:
   GoSub SetUp
   GoSub Simulate
End
//====================================================================
//====================================================================
SetUp:
   flip on
   wc = blue
   Cx = 400 \ Cy = 300
   Cr = 200 \ CCr = Cr-10 \ Cr2 = 100
   IndR = Cr-3 \ IrR = Cr-15 \ IrRR = 4
   DegPerSpoke=20
   tt = dtor(90+DegPerSpoke/4.0)
   Ir1x = Cx+IrR*cos(tt) \ Ir1y = Cy-IrR*sin(tt)
   tt = dtor(90-DegPerSpoke/4.0)
   Ir2x = Cx+IrR*cos(tt) \ Ir2y = Cy-IrR*sin(tt)
   LineWidth 2
   data Colors;wc,white
   gosub DrawBase
   SetColor White,Cyan
   xyText 90,10," Quadrature Encoder Simulator ","Lucida Handwriting",25,fs_Bold
   setcolor black,white
   xyText 10,120,"  State:","",15,fs_Bold
   xyText 10,150,"Counter=","",15,fs_Bold
   xyText 10,180,"  Angle=","",15,fs_Bold
   xyText 10,450,"Act.Angle=","",15,fs_Bold
   xyText 500,510,"Degrees Per Spoke","",15,fs_Bold
   xyText 500,510,"D","",15,fs_Bold | fs_Underlined
   s = "Min 1, Max 90 and has to divide into 360 and the result must be even"
   xyText 500,540,s,"Time New Roman",8
   s = "Unsuitable IR placement causes errors in count"
   xyText 500,560,s,"Time New Roman",8
   Rectangle 710,510,750,535
   xystring 713,513,format(DegPerSpoke,"##0")
   SaveScr 0,0,800,500
   data Buttons;"&Instructions","&Reset"
   AddButton Buttons[0],10,570,105
   AddButton Buttons[1],115,570   
   GoSub Initialize
   GoSub Instructions
   flip
Return
//====================================================================
Initialize:
   rDPS = dtor(DegPerSpoke) \ dA = DegPerSpoke/2.0
   oTheta = -1
   Theta = 0
   FirstTime = true
Return
//====================================================================
Simulate:
   while true
     //---get user Input
     GoSub CheckInputs
     if keydown(kc_LArrow) || mb==1 then Theta = Theta-1
     if keydown(kc_RArrow) || mb==2 then Theta = Theta+1
     If Theta < 0 then Theta = Theta+360
     if Theta >= 360 then Theta = Theta-360
     if oTheta != Theta
        oTheta = Theta
        //-----redraw wheel
        Restorescr
        j = 0
        for i=0 to (360/DegPerSpoke)-1
           pie Cx-CCr,Cy-CCr,Cx+CCr,Cy+CCr,dtor(450-Theta-dA)+i*rDPS,rDPS,Colors[j],Colors[j]
           j = (j+1)#2
        next
        circle Cx-Cr2,Cy-Cr2,Cx+Cr2,Cy+Cr2,wc,wc
        Circle Cx-10,Cy-10,Cx+10,Cy+10,white,white
        gosub Quadrature  //---Calculate Quadrature Count
        //---redraw IRs
        circle Ir1x-IrRR,Ir1y-IrRR,Ir1x+IrRR,Ir1y+IrRR,red*IR1,red*IR1
        circle Ir2x-IrRR,Ir2y-IrRR,Ir2x+IrRR,Ir2y+IrRR,lightred*IR2,lightred*IR2
        //---draw indicator
        rTheta = dtor(450-Theta)
        dx = Cx+IndR*Cos(rTheta) \ dy = Cy-IndR*Sin(rTheta)
        circle dx-2,dy-2,dx+2,dy+2,white,white
        Angle = round(frac(Counter*dA/360.0)*36000)/100.0
        if Angle < 0 then Angle = Angle+360
        xyString 120,183,Angle
        xyString 135,453,Theta
        flip
     endif
   wend
Retrun
//====================================================================
Quadrature:
   //---read the IRs
   readpixel Ir1x,Ir1y,IR1
   readpixel Ir2x,Ir2y,IR2
   if IR1 = Blue then IR1 = 0
   If IR1 = White then IR1 = 1
   if IR2 = Blue then IR2 = 0
   If IR2 = White then IR2 = 1
   State = (IR1 << 1)+IR2   //---Calculate Current State
   dCount = -2
   if FirstTime
      FirstTime = false
      Counter = 0
      dCount = 0
   else      
      if oState = 0
         if State = 1 then dCount = -1
         if State = 2 then dCount = 1
         if State = 0 then dCount = 0
      elseif oState = 1
         if State = 3 then dCount = -1
         if State = 0 then dCount = 1
         if State = 1 then dCount = 0
      elseif oState = 2
         if State = 0 then dCount = -1
         if State = 3 then dCount = 1
         if State = 2 then dCount = 0
      else
         if State = 2 then dCount = -1
         if State = 1 then dCount = 1
         if State = 3 then dCount = 0
      endif
   endif   
   oState = State
   if dCount > -2
      Counter = Counter+dCount
   else
      xyText 650,180,"State Error","",15,fs_Bold
      circle 700,200,710,210,Red,red
   endif
   xyString 120,123,IR1,IR2," = ",State
   xyString 120,153,Counter
Return
//====================================================================
CheckInputs:
  getbutton btn
  if btn == Buttons[0] then n=MsgBox(IM)
  if btn == Buttons[1] then GoSub Initialize \ Return
  readmouse x,y,mb
  if (mb == 1 && within(x,710,750) && within(y,510,535)) || KeyDown(Ascii("D"))
     mb = 0
     s = StrInput(Msgs[0],Msgs[1],""+DegPerSpoke)
     if IsNumber(s)
        s = Limit(ToNumber(s),1,90)
        if s == DegPerSpoke  then return
        While (360#s > 0) || (360/s)#2 > 0)
          s = s+1
        wend  
        DegPerSpoke = s
        xystring 713,513,format(DegPerSpoke,"##0")
        GoSub Initialize
     endif
  elseif mb==11 && PolarR(x-Cx,y-Cy) < Cr
     Ir1x = x \ Ir1y = y   
     GoSub Initialize
  elseif mb==12 && PolarR(x-Cx,y-Cy) < Cr
     Ir2x = x \ Ir2y = y   
     GoSub Initialize
  endif
Return
//====================================================================
DrawBase:
  LineWidth 2
  DB_d1 = 207 \ DB_d2 = DB_d1 - 57
  for i=0 to 359 step 10
     DB_R = DB_d1 \ DB_p = false
     if !(i#30) then DB_R = DB_d1+8 \ DB_p = true
     if !(i#90) then DB_R = DB_d1+20
     dx1 = DB_R*cos(dtor(i-90)) \ dy1 = DB_R*sin(dtor(i-90))
     dx2 = DB_d2*cos(dtor(i-90)) \ dy2 = DB_d2*sin(dtor(i-90))
     line Cx+dx2,Cy+dy2,Cx+dx1,Cy+dy1
     if i == 0  then ddx = -5 \ ddy = -7
     if within(i,1,90) then ddx = 2 \ ddy = -7
     if within(i,91,179)  then ddx = 2 \ ddy = 1
     if within(i,180,270) then ddx = -15 \ ddy = -1
     if within(i,270,360) then ddx = -25 \ ddy = -7
     if DB_p then xytext Cx+dx1+ddx,Cy+dy1+ddy,tostring(i),"",-1,fs_bold
  next
  circle Cx-Cr,Cy-Cr,Cx+Cr,Cy+Cr,wc,wc
Return
//=====================================================================
Instructions:
  data Msgs;"QuadratureSimulator.Bas"
  data Msgs;"Enter number of Degrees Per Spoke (1 to 90):"
  data IM;Msgs[0]
  data IM;"This program helps in visualizing a Qudrature Encoding System. To"
  data IM;"move the wheel use the left or right arrow keys or the click the"
  data IM;"left or right mouse buttons.",""
  data IM;"A quadrature encoder measures movement and direction in incremental"
  data IM;"steps of a resolution dependant on the parameters of the system.",""
  data IM;"The principle is that two sensors are set up so that a series of"
  data IM;"tabs will obstruct the sensors in a certain sequence. This sequence"
  data IM;"will be different if the movement is left to right than if it is"
  data IM;"right to left (linear or rotational).",""
  data IM;"The encoder uses the sequences to determine the direction of"
  data IM;"movement and also maintains a count of the incremental movements."
  data IM;"The count will be a negative, 0, or positive number, depending"
  data IM;"on the cumulative sum of the movements that occurred and in what"
  data IM;"direction.",""
  data IM;"To use the encoder you need to know its resolution per incremental"
  data IM;"move. With this information and knowing the count you will be able"
  data IM;"to calculate the absolute amount of the movement. For example for a "
  data IM;"rotational encoder if the resolution is 3 degrees then a count of "
  data IM;"50 means a rotation of 150 degrees and a count of -10 means a "
  data IM;"rotation of 30 degrees in the other direction. The count is a total"
  data IM;"count. The count of 50 could have been 10 steps left then 60 to "
  data IM;"the right (-10+60 = 50). It is up to the user to keep continuous"
  data IM;"track if the movement if that is of concern.",""
  data IM;"An encoder can use Infrared sensors where the tabs block the "
  data IM;"transmitter from the receiver as they move. Another sensor that can"
  data IM;"be used is the Hall Effect sensor that measure magnetic flux (the"
  data IM;"the tabs can be metallic teeth like in a gear). The ultimate outcome"
  data IM;"is that each sensor will be either ON or OFF depending on the position"
  data IM;"of the tabs. If the sensors are placed such that the on/off sequence is "
  data IM;"90 degrees out of phase the movement will create a sequence of:"
  data IM;"   00, 10, 11, 01 for right moves and 00, 01, 11, 10 for left moves."
  data IM;"    0-2-3-1-0 (right)             and   0-1-3-2-0 (left)",""
  data IM;"The encoder monitors these states and determines the direction"
  data IM;"depending on the sequence of state changes. Also for each change"
  data IM;"the count is incremented or decremented according to the direction.",""
  data IM;"This is why it is called a Quadrature Encoder (4 states encoding).",""
  data IM;"The placement of the sensors greatly affects the encoder and also "
  data IM;"the spacing of the tabs affects the resolution. Additionally the"
  data IM;"speed of the movement has to be less than the speed of the encoding"
  data IM;"circuitry so that a phase change cannot occur before the encoder"
  data IM;"is able to detect the sensors' change of state and process it to "
  data IM;"increment/decrement the count. In other words the speed of the "
  data IM;"physical movement cannot be any faster than the speed of the "
  data IM;"electronic circuitry.",""
  data IM;"In the simulation you can define the size of the tabs (degrees per"
  data IM;"spoke) and the placement of the sensors.",""
  data IM;"The sensors will blink on/off to indicate if they are blocked or"
  data IM;"not and the data displayed on the left of the screen will indicate"
  data IM;"the current state, the count, and since this is a rotational system"
  data IM;"a number indicating the angle as calculated by the multiplication"
  data IM;"of the count with the resolution. The resolution is calculated from"
  data IM;"the size of the tabs (spokes). Also the angle is adjusted so as to"
  data IM;"be a compass heading (0 to 360). Additionally the actual angle"
  data IM;"is displayed. The actual angle is how much the wheel actually moved"
  data IM;"in contrast with the calculated movement which will be less"
  data IM;"accurate, depending on the resolution.",""
  data IM;"To change the tab size (degrees per spoke) click on the field"
  data IM;"and enter the amount when prompted to do so. The final number"
  data IM;"actually used is adjusted from your entered number to make it"
  data IM;"an allowable value (if it is not). The allowable value has to"
  data IM;"devide into 360 and has to result into an even value, since the"
  data IM;"number of spokes will be 360/(degrees per spoke)/2.",""
  data IM;"To change the placement of the sensors use the Shift+Left mouse"
  data IM;"click to reposition the left sensor(red) and Shift+Right mouse"
  data IM;"click to reposition the right sensor (orange). You are restricted"
  data IM;"to placing the sensors on the wheel, but any where you want. It"
  data IM;"is up to you to place them in a sensible place within the wheel.",""
  data IM;"If you place the sensors incorrectly for the tab size, the "
  data IM;"sequence will be incorrect and the count will be incorrect.",""
  data IM;"If you get an incorrect situation an LED will light up on the"
  data IM;"left side of the screen next to the wheel to indicate the fact.",""
  data IM;"When you change the tab size or the sensor placements the system"
  data IM;"will be rest.",""
  data IM;"The sensors start being placed so as to share a tab. This is so"
  data IM;"you can easily view the 90 degrees out of phase relationship. But,"
  data IM;"in a real life system, sensors placed so closely together might"
  data IM;"interfere together. In a physical system you would have to place"
  data IM;"the sensors on widely spaced tabs, but still maintain the same"
  data IM;"phase relationship. Experiment with placing the sensors manually"
  data IM;"to see how this works."
  //n=MsgBox(IM)
Return
//=====================================================================
