Main:
  // The following code represents a simple walking algorithm
  // (as described in a SERVO article by Blankenship/Mishal)
  // We suggest you write your own StartWalk, LtStep, RtStep, and StopWalk
  // routines to see if you can create your own algorith....
  //    TRY IT BEFORE YOU LOOK AT OURS IN DETAIL
  // Examine StartWalk (and others) to see the basic
  // principles for controlling the man.
  // A group of primitive Subroutines (such as BodyForward,LtLegBend etc)
  // have been created to make programming the man easier.  You can create 
  // others that you want are need (see how we did ours).
  Flip on
  gosub Init
    while True
    gosub StartWalk
    repeat
      gosub LtStep
      gosub RtStep
    until Man[4,1]>650
    gosub StopWalk
    delay 1000
    gosub CreateSurface
  wend
end

StartWalk:
  MainFoot=Rt // sets initial balance foot
  // the most forward foot TOUCHING the ground automaticaly becomes the balance foot
  for k=1 to 15
    if k>7 then gosub LtArmForward
    gosub BodyForward
    gosub LtLegBend
    gosub DrawMan // MUST be called in EVERY loop to SEE the action
                  // It also lets the man fall if he is off balanced
  next
  for k=1 to 2
    gosub LtKneeBackward
    gosub DrawMan
  next
  // save these angle so man can return to this position for normalization
  MyLtThAngle=LtThAngle
  MyLtCaAngle=LtCaAngle
  MyRtThAngle=RtThAngle
  MyRtCaAngle=RtCaAngle
return

StopWalk:
  for k=1 to 30
    if Tilt<>Vertical then gosub BodyBackward
    gosub LtLegStraighten
    gosub DrawMan
  next
return

RtStep:  
  for k=1 to 10
    while Tilt<>Forward
      gosub BodyForward
      gosub DrawMan
    wend
    gosub RtLegForward
    gosub LtArmForward
    gosub RtArmBackward
    gosub LtKneeForward
    gosub DrawMan
  next
  cnt=0
  while MainFoot=Lt
    cnt=cnt+1
    gosub RtLegStraighten
    if Tilt=Balanced then gosub BodyForward  
    gosub DrawMan
    if cnt>7
      while MainFoot=Lt
        gosub BodyBackward
        gosub LtLegBend
        gosub DrawMan
      wend
    endif
  wend
  repeat
    Counting=False
    if RtThAngle>MyRtThAngle then gosub RtThighBackward\Counting=True
    if RtCaAngle>MyRtCaAngle then gosub RtKneeForward\Counting=True
    if LtThAngle<MyLtThAngle then gosub LtLegForward\Counting=True
    if LtCaAngle>MyLtCaAngle then gosub LtKneeForward\Counting=True
    if Tilt=Forward then gosub BodyBackward
    gosub DrawMan
  Until !Counting
return

LtStep:
  for k=1 to 10
    while Tilt<>Forward
      gosub BodyForward
      gosub DrawMan
    wend
    gosub LtLegForward
    gosub RtArmForward
    gosub LtArmBackward
    gosub RtKneeForward
    gosub DrawMan
  next
  cnt=0
  while MainFoot=Rt
    cnt=cnt+1
    gosub LtLegStraighten
    if Tilt=Balanced then gosub BodyForward
    gosub DrawMan    
    if cnt>7
      while MainFoot=Rt
        gosub RtLegBend
        gosub BodyBackward
        gosub DrawMan
      wend
    endif
  wend
  repeat
    Counting=False
    if LtThAngle>MyRtThAngle then gosub LtThighBackward\Counting=True
    if LtCaAngle>MyRtCaAngle then gosub LtKneeForward\Counting=True
    if RtThAngle<MyLtThAngle then gosub RtLegForward\Counting=True
    if RtCaAngle>MyLtCaAngle then gosub RtKneeForward\Counting=True  
    if Tilt=Forward then gosub BodyBackward
    gosub DrawMan
  Until !Counting
return



Calculate:
  if !FirstTime
    // is man off balance
    if Within(-Man[5,1]+Man[MainFoot,3],0,5)
          Tilt=Vertical
    elseif Within(+Man[5,1]-Man[MainFoot,3],5,20)
          Tilt=Balanced
    elseif Man[5,1]<Man[MainFoot,3]
      // fall backwards some
      Tilt=Backward
      for ii=0 to 5
        Man[Joints[ii],9]=Man[Joints[ii],9]+.2
      next
    else
      // fall forwards some
      Tilt=Forward
      for ii=0 to 5
        Man[Joints[ii],9]=Man[Joints[ii],9]-.2
      next
    endif
  endif
  FirstTime=False
  for j=0 to 9 
    for i=0 to 9
      if Man[i,0]=j then break
    next
    // i now points to the line to calculate
    // if this is NOT the first line, we must get starting x,y
    if j<>0
      Man[i,1]=Man[Man[i,11],3]
      Man[i,2]=Man[Man[i,11],4]
    endif
    // now calculate the new ending point
    Man[i,3]=Man[i,1]+Man[i,8]*cos(DtoR(Man[i,9]))
    Man[i,4]=Man[i,2]-Man[i,8]*sin(DtoR(Man[i,9]))
  next 
  // now make sure the "new" man is at the same "bottom"
  // first set current bottom (CurY)
  CurY=Man[MainFoot,4]
  // adjust all Y values
  for i=0 to 9
    Man[i,2]=Man[i,2]-(CurY-LastY)
    Man[i,4]=Man[i,4]-(CurY-LastY)
  next
  // then adjust the horizontal position
  CurX=Man[MainFoot,3]
  for i=0 to 9
    Man[i,1]=Man[i,1]-(CurX-LastX)
    Man[i,3]=Man[i,3]-(CurX-LastX)
  next  
  // easy parameters that user can use when "programming" the robot
  // add parameters to fit your needs (place also in Init:)
  LtThAngle=Man[3,9]
  LtCaAngle=Man[0,9]
  RtThAngle=Man[7,9]
  RtCaAngle=Man[6,9]
  LtX=Man[0,3]   // x position of the left foot
  RtX=Man[6,3]   // x position of the right foot
  BodyX=Man[5,3] // x position of the lower body
return

DrawMan:
  gosub Calculate
  RestoreScr
  LineWidth 2
  for i= 0 to 9
    if Man[i,6]<>0
      t=Man[i,6]
      circle Man[i,1]-t,Man[i,2]-t,Man[i,1]+t,Man[i,2]+t,Black,Man[i,10]
    endif
    if Man[i,7]<>0
      t=Man[i,7]
      if !(i=0 or i=6)
        circle Man[i,3]-t,Man[i,4]-t,Man[i,3]+t,Man[i,4]+t,Black,Man[i,10]
      else
        circle Man[i,3]-t,Man[i,4]-t,Man[i,3]+3*t,Man[i,4]+t,Black,Man[i,10]
      endif
    endif
    Line Man[i,1],Man[i,2],Man[i,3],Man[i,4],Man[i,5],Man[i,10]
  next
  Flip
  // set MainFoot
  // see if the feet are ON the ground (YOU must modify if size of man changes)
  ReadPixel Man[0,3]+15,Man[0,4]+7,LtCheck
  ReadPixel Man[6,3]+15,Man[6,4]+7,RtCheck
  //print LtCheck;RtCheck;Man[0,3];Man[6,3]\flip\delay 2000
  if LtCheck=Green and RtCheck=Green 
    // both on ground, see which is forward
    if !Changed
      // don't make any changes
    elseif Man[0,3]<Man[6,3]
      MainFoot=Rt
      Changed=False
    else
      MainFoot=Lt
      Changed=False
    endif
  else
    // only one is touching
    if LtCheck=Green
      MainFoot=Lt
      Changed=True
    elseif RtCheck=Green
      MainFoot=Rt
      Changed=True
    // no check for at least one foot down... 
    endif
  endif
  //MainFoot=Rt
  LastY=Man[MainFoot,4]
  LastX=Man[MainFoot,3]
return

Init:
  // examine the variables here, especially the array to see
  // how the man's parameters are organized
  FirstTime=True
  MediumGray = 6381921
  Lt=0
  Rt=6
  MainFoot=Rt
  LtDn=True
  RtDn=True
  Changed=False
  Forward=1
  Backward=2
  Balanced=3
  Vertical=4
  IA=2 // incr.angle for movements
  // these Joints are effected by tilt
  Data Joints; 0,3,4,5,6,7
  Dim Man[10,12]
  // first index as indicated below
  // note: there are in the DRAWING order
  //   0-left calf
  //   1-left forearm
  //   2-left bicept
  //   3-left thigh
  //   4-neck
  //   5-body
  //   6-right calf
  //   7-right thigh
  //   8-right bicept
  //   9-right forearm
  // second index as indicated below
  // these items refer to the body part indicated by the first index
  //   0-order that the calculations must be performed
  //   1-starting x position of this body part
  //   2-starting y position of this body part
  //   3-ending x position of this body part
  //   4-ending y position of this body part
  //   5-line width for this body part
  //   6-starting circle radius
  //   7-ending circle radius
  //   8-length of this part
  //   9-angle (from due east)
  //  10-color
  //  11-use the ending x,y of the part specified here as the 
  //     starting x,y for THIS part
  // now we must initialize the elements of the array.
  Man[4,1]=56 \ Man[4,2]=300 // starting position of the man's head
  // now set up calculation order
  Man[0,0]=9
  Man[1,0]=4
  Man[2,0]=2
  Man[3,0]=7
  Man[4,0]=0
  Man[5,0]=1
  Man[6,0]=8
  Man[7,0]=6
  Man[8,0]=3
  Man[9,0]=5
  // set up line widths
  Man[0,5]=10
  Man[1,5]=8
  Man[2,5]=9
  Man[3,5]=12
  Man[4,5]=10
  Man[5,5]=20
  Man[6,5]=10
  Man[7,5]=12
  Man[8,5]=9
  Man[9,5]=8
  // set up first circle radius
  Man[0,6]=8
  Man[1,6]=6
  Man[2,6]=0
  Man[3,6]=10
  Man[4,6]=14
  Man[5,6]=12
  Man[6,6]=8
  Man[7,6]=10
  Man[8,6]=8
  Man[9,6]=6
  // set up second circle radius
  Man[0,7]=6
  Man[1,7]=7
  Man[2,7]=0
  Man[3,7]=0
  Man[4,7]=0
  Man[5,7]=12
  Man[6,7]=6
  Man[7,7]=0
  Man[8,7]=0
  Man[9,7]=7
  // set up length of this part
  Man[0,8]=50
  Man[1,8]=40
  Man[2,8]=40
  Man[3,8]=50
  Man[4,8]=30
  Man[5,8]=70
  Man[6,8]=50
  Man[7,8]=50
  Man[8,8]=40
  Man[9,8]=40
  // set up angle of this part
  Man[0,9]=-90
  Man[1,9]=-70
  Man[2,9]=-90
  Man[3,9]=-90
  Man[4,9]=-90
  Man[5,9]=-90
  Man[6,9]=-90
  Man[7,9]=-90
  Man[8,9]=-90
  Man[9,9]=-70
  // set up color of this part
  Man[0,10]=MediumGray
  Man[1,10]=MediumGray
  Man[2,10]=MediumGray
  Man[3,10]=MediumGray
  Man[4,10]=MediumGray
  Man[5,10]=Black
  Man[6,10]=DarkGray
  Man[7,10]=DarkGray
  Man[8,10]=Gray
  Man[9,10]=Gray
  // set up link to part with starting coordinates
  Man[0,11]=3
  Man[1,11]=2
  Man[2,11]=4
  Man[3,11]=5
  Man[4,11]=0
  Man[5,11]=4
  Man[6,11]=7
  Man[7,11]=5
  Man[8,11]=4
  Man[9,11]=8
  gosub CreateSurface
  LastX=50
  LastY=502
  gosub DrawMan
return

// Arm movements ------------

RtArmForward:
  // move forearm more
  Man[8,9]=Man[8,9]+IA
  Man[9,9]=Man[9,9]+IA*2
return

RtArmBackward:
  // more forearm more
  Man[8,9]=Man[8,9]-IA
  Man[9,9]=Man[9,9]-IA*2
return

LtArmForward:
  // move forearm more
  Man[2,9]=Man[2,9]+IA
  Man[1,9]=Man[1,9]+IA*2
return

LtArmBackward:
  // move forearm more
  Man[2,9]=Man[2,9]-IA
  Man[1,9]=Man[1,9]-IA*2
return

// Leg movements ------------

RtLegForward:
  // maintains calf angle relative to thigh
  Man[7,9]=Man[7,9]+IA
  Man[6,9]=Man[6,9]+IA
return

RtThighForward:
  // maintain calf angle relative to ground
  Man[7,9]=Man[7,9]+IA
return

RtThighBackward:
  // maintain calf angle relative to ground
  if Man[7,9]<Man[6,9] then Man[6,9]=Man[6,9]-IA
  Man[7,9]=Man[7,9]-IA
return

RtKneeForward: 
  // or right foot backward, depending on pivot point
  Man[6,9]=Man[6,9]-IA
return

RtKneeBackward: 
  // or right foot forward, depending on pivot point
  Man[6,9]=Man[6,9]+IA
return

RtLegBackward:
  // maintains calf angle relative to thigh
  Man[7,9]=Man[7,9]-IA
  Man[6,9]=Man[6,9]-IA
return

LtLegForward:
  // maintains calf angle relative to thigh
  Man[3,9]=Man[3,9]+IA
  Man[0,9]=Man[0,9]+IA
return

LtThighForward:
  // maintains calf angle relative to ground
  Man[3,9]=Man[3,9]+IA
return

LtThighBackward:
  // maintains calf angle relative to ground
  if Man[3,9]<Man[0,9] then Man[0,9]=Man[0,9]-IA
  Man[3,9]=Man[3,9]-IA
return

LtKneeForward:
  //or left foot backward depending on pivot point
  Man[0,9]=Man[0,9]-IA
return

LtKneeBackward:
  //or left foot forward depending on pivot point
  Man[0,9]=Man[0,9]+IA
return



LtLegBackward:
  // maintains calf angle relative to thigh
  Man[3,9]=Man[3,9]-IA
  Man[0,9]=Man[0,9]-IA
return

LtLegBend:
  // or lift if not the support leg
  Man[3,9]=Man[3,9]+IA
  Man[0,9]=Man[0,9]-IA
return

LtLegStraighten:
  // make sure foot does not pass knee
  if Man[0,9]<Man[3,9]
    Man[3,9]=Man[3,9]-IA
    Man[0,9]=Man[0,9]+IA
  endif
return

RtLegBend:
  // or lift if not the support leg
  Man[7,9]=Man[7,9]+IA
  Man[6,9]=Man[6,9]-IA
return

RtLegStraighten:
  // make sure foot does not pass knee
  if Man[6,9]<Man[7,9]
    Man[7,9]=Man[7,9]-IA
    Man[6,9]=Man[6,9]+IA
  endif
return

// Body movements -----------

BodyForward:
  Man[4,9]=Man[4,9]-IA/4.0
  Man[5,9]=Man[5,9]-IA/4.0
  //Man[4,1]=Man[4,1]+IA
return

BodyBackward:
  Man[4,9]=Man[4,9]+IA/4.0
  Man[5,9]=Man[5,9]+IA/4.0
return

// Head movements -----------
  
HeadForward:
  Man[4,9]=Man[4,9]-IA
return

HeadBackward:
  Man[4,9]=Man[4,9]+IA
return

//---------------------------

CreateSurface:
  ClearScr
  Line 0,513,150,513,10,Green
  Lineto Random(150)+300,525-Random(75),10,Green
  Lineto Random(150)+500,513-Random(50),10,Green
  Lineto 800,513,10,Green
  SaveScr
  // reset Man parameters
  Man[4,1]=56 \ Man[4,2]=300 // starting position of the man's head
  FirstTime=True
  LastX=50
  LastY=502
  LtDn=True
  RtDn=True
  Man[0,9]=-90
  Man[1,9]=-70
  Man[2,9]=-90
  Man[3,9]=-90
  Man[4,9]=-90
  Man[5,9]=-90
  Man[6,9]=-90
  Man[7,9]=-90
  Man[8,9]=-90
  Man[9,9]=-70
  gosub Calculate
return

ColorSelection:
  // This is a utility subroutine used to find
  // a custom color for some of the joints
  ccc=PromptColor(0)
  print ccc
  data cc; black, darkgray, ccc
  for i=1 to 3    
    rectangle (i*100),10,(i*100)+100,100,cc[i-1],cc[i-1]
  next
return

