MainProgram:
  GoSub Initialization
  GoSub MonitorInputs
End
//================================================
Initialization:
  DeviceNum = 0 \ LED_Pin = 5 \ LED_On = 1 \ PB_Pin = 6 \ PB_Count = 0
  LCDText = "LCDText" \ LCDData = "LCDData" \ LCDCmd = "LCDCmd"
  AddEdit LCDText,160,430,200,0,"Testing","Enter text to display on the LCD"
  AddEdit LCDCmd,160,460,50,0,"0x01","Enter the LCD command in hex or decimal"
  AddEdit LCDData,160,490,50,0,"0x30","Enter the LCD data in hex or decimal"
  IntegerEdit LCDCmd \ IntegerEdit LCDData
  InstBtn = "Instructions" \ InitLCDBtn = "Init LCD" \ SndTxtBtn = "Send Text To LCD"
  SndCmdBtn = "Send LCD Command" \ SndDataBtn = "Send Data To LCD"
  data Buttons;InitLCDBtn   ,190,360,100,60,"Initialize the LCD"
  data Buttons;SndTxtBtn    ,10,430,135,0,"Send the Text String to the LCD"
  data Buttons;SndCmdBtn    ,10,460,135,0,"Send the Command byte to the LCD"
  data Buttons;SndDataBtn   ,10,490,135,0,"Send The Data byte to the LCD"
  for n=0 to MaxDim(Buttons,1)-1 step 6
    AddButton Buttons[n],Buttons[n+1],Buttons[n+2],Buttons[n+3],Buttons[n+4],Buttons[n+5]
  next
  Gosub Display_DllSpecs
  GoSub Display_DeviceSpecs
  GoSub Init_LCD
  addtimer "t1",150 \ onTimer tHandler
Return
//================================================
tHandler:
  lt = LastTimer()
  n = usbm_WriteABit(DeviceNum,255-2^LED_Pin,2^LED_Pin*LED_On) 
  LED_On = !LED_On   
  GoSub Check_PushButton
  onTimer tHandler
Return
//================================================
MonitorInputs:
  while true
    GetButton btn
    if btn == InitLCDBtn
      GoSub Init_LCD
    elseif btn == SndTxtBtn
      S = GetEdit(LCDText)
      n = usbm_LCDCmd(DeviceNum, 0x1)  //clear LCD
      GoSub Write_LCD
    elseif btn == SndCmdBtn
      n = ToNumber(GetEdit(LCDCmd),0)
      n = usbm_LCDCmd(DeviceNum,n )
    elseif btn == SndDataBtn
      n = ToNumber(GetEdit(LCDData),0)
      n = usbm_LCDData(DeviceNum,n )
    endif
  wend
Return
//================================================
Display_DllSpecs:
  xyText 20,0,"USB Micro DLL Data","Times New Roman",15,fs_Bold|fs_Underlined
  m = usbm_DLLSpecs()
  xyText 10,30,Extract(m,"|",1),"Times New Roman",10
  xyText 10,45,Extract(m,"|",2),"Times New Roman",10
  xyText 10,60,"Version#:"+Extract(m,"|",4)+":"+Extract(m,"|",3),"Times New Roman",10
Return
//================================================
Display_DeviceSpecs:
  RectangleWH 0,75,230,120,white,white
  n = usbm_finddevices()
  n = usbm_numberofdevices()
  m = ""
  if n != 1 then m = "s"
  xyText 10,79, spaces(20)+"Found "+n+" Device"+m,"Times New Roman",10,fs_Bold
  xyText 20,95,"Device Specifications","Times New Roman",15,fs_Bold|fs_Underlined
  if usbm_DeviceValid(DeviceNum)
    n = usbm_initports(DeviceNum)
    n = usbm_DirectionA(DeviceNum,255-2^PB_Pin,255) //all outputs but PB+Pin is set as input
    n = usbm_SetBit(DeviceNum,PB_Pin)               //with internal pull-up resistor
    n = usbm_DirectionB(DeviceNum,255,255)     
    m = usbm_DeviceSpecs(DeviceNum)        
    VID = "Made By:"+hex(tonumber(extract(m,"|",3)))+":"
    MFR = VID+extract(m,"|",4)
    PID = "Device:"+hex(tonumber(extract(m,"|",2)))+":"
    PRD = PID+extract(m,"|",5)
    DID = "Version:"+hex(tonumber(extract(m,"|",1)))
    SER = "S/N:"+extract(m,"|",6)
  else
    MFR = "Made By:" \ PRD = "Device:"\ DID = "" \ SER = "S/N:"
  endif
  xyText 10,125,MFR,"Times New Roman",10
  xyText 10,140,PRD+spaces(20)+DID,"Times New Roman",10
  xyText 10,155,SER,"Times New Roman",10
Return
//================================================
Check_PushButton:
  if !usbm_DeviceValid(DeviceNum) then return
  n = usbm_ReadA(DeviceNum) & (2^PB_Pin)
  if !n //---active low Push button so we act on 0
    PB_Count = PB_Count+1 \ S = ""+PB_Count
    n = usbm_LCDCmd(DeviceNum, 0x1)  //clear LCD
    GoSub Write_LCD      
  endif
Return
//=========================================================
Init_LCD:
  if !usbm_DeviceValid(DeviceNum) then return
  n = usbm_InitLCD(DeviceNum, 0x12, 0x13)   //---RW=A1,RS=A2,E=A3,Data=Port B     
  n = usbm_SetBit(DeviceNum,0)  \ delay 20  //---RES connected to A0
  n = usbm_ResetBit(DeviceNum,0)\ delay 100 //High-Low-High to reset with 100 ms pulse
  n = usbm_SetBit(DeviceNum, 0) \ delay 100 //give the LCD time to settle
  n = usbm_LCDCmd(DeviceNum, 0x1C)  //---command sequence to setup the LCD
  n = usbm_LCDCmd(DeviceNum, 0x14)  //---this is obtainable from the device
  n = usbm_LCDCmd(DeviceNum, 0x28)  //---specs sheet
  n = usbm_LCDCmd(DeviceNum, 0x4F)  //
  n = usbm_LCDCmd(DeviceNum, 0xE0)  //
  n = usbm_LCDCmd(DeviceNum, 0x1)   //--clear the LCD display
Return
//=========================================================
Write_LCD: //S is the string to write
  if !usbm_DeviceValid(DeviceNum) then return
  n = usbm_LCDCmd(DeviceNum, 0x1)  //code 0x1 clears the LCD
  if Length(S) == 0 then Return
  for i = 1 To Length(S)    
    n=usbm_LCDData(DeviceNum,GetStrByte(S,i)) //send the ascii code of each character
  next					     
Return
//=========================================================
